@@ -150,7 +150,7 @@ impl rustyline::Helper for Helper {}
 #[cfg(test)]
 mod tests {
     use super::*;
-    use nu_engine::basic_evaluation_context;
+    use nu_engine::EvaluationContext;
     use rustyline::completion::Completer;
     use rustyline::line_buffer::LineBuffer;
 
@@ -164,7 +164,7 @@ mod tests {
         buffer.insert_str(0, text);
         buffer.set_pos(text.len() - 1);
 
-        let helper = Helper::new(basic_evaluation_context().unwrap(), None);
+        let helper = Helper::new(EvaluationContext::basic(), None);
 
         helper.update(&mut buffer, "cd ".len(), &replacement);
 
@@ -184,7 +184,7 @@ mod tests {
         buffer.insert_str(0, text);
         buffer.set_pos(text.len() - 30);
 
-        let helper = Helper::new(basic_evaluation_context().unwrap(), None);
+        let helper = Helper::new(EvaluationContext::basic(), None);
 
         helper.update(&mut buffer, "cd ".len(), &replacement);
 
